package com.purui.ghip.service;

import cn.hutool.http.HttpRequest;
import com.alibaba.fastjson.JSON;
import com.purui.ghip.model.MapInterface;
import com.purui.ghip.model.ResultSet;
import com.purui.ghip.model.XYSHisPlatformMaintain;
import com.purui.ghip.utils.RedisUtil;

import lombok.extern.slf4j.Slf4j;
import org.springframework.http.HttpHeaders;
import org.springframework.stereotype.Service;
import org.springframework.web.reactive.function.BodyInserters;
import org.springframework.web.reactive.function.client.WebClient;
import reactor.core.publisher.Mono;
import java.util.List;

/**
 * 转发服务实现
 */
@Service
@Slf4j
public class HealthService implements IHealthService{

	private  final RedisUtil redisUtil;

	public HealthService(RedisUtil redisUtil) {
		this.redisUtil = redisUtil;
	}

	public String health() {
		/**
		 * 1.服务启动后从redis获取接口配置，如果为空则从2.6加载所有接口配置（webclient/restTemplate），同时缓存接口配置到redis
		 * 1.1 支持缓存定期更新，支持缓存失效后重新加载
		 * 2.根据接口请求地址，在接口配置中找转发的服务地址
		 * 3.转发请求（webclient/restTemplate）
		 * 4.返回结果
		 */
		return "ok";
	}

	/**
	 * 获取接口转发规则
	 * @param hospCode   his医院代码
	 * @return 平台规则返回的字符串
	 */
	public String getInterfaceForwardingRules(String hospCode){
		String checkKey="XYSHisPlatformMaintain:"+hospCode;
		String rulesInfo=redisUtil.get(checkKey);
		if ((rulesInfo=="")||(rulesInfo==null)){
			//获取不到重新查询上传
			String params="{\"params\":[{}],\"code\":\"012\",\"session\":[{}]}";
			ResultSet rulesResult = WebClient.create()
					.post()
					.uri("http://172.16.2.6:52773/External/hipExternalMutiple")
					.header("Authorization", "Basic cHJoaXA6cHJoaXBAMjAyMA==")
					.body(BodyInserters.fromObject(params))
					.retrieve()
					.bodyToMono(ResultSet.class)
					.block();
			String errorCode=rulesResult.getErrorCode();
			if ((errorCode!="0")||(errorCode==null)){
				return "";
			}
			List<XYSHisPlatformMaintain> XYSHisPlatformMaintainArr= (List<XYSHisPlatformMaintain>) rulesResult.getResult();
			for (XYSHisPlatformMaintain xysHisPlatformMaintain : XYSHisPlatformMaintainArr) {
				String curHospCode=xysHisPlatformMaintain.getHospCode();
				String curCheckKye="XYSHisPlatformMaintain:"+curHospCode;
				redisUtil.set(curCheckKye,xysHisPlatformMaintain.toString());
			}
			rulesInfo=redisUtil.get(checkKey);
		}
		return rulesInfo;
	}


	/**
	 * 获取接口配置
	 * @param interfaceCode   接口代码
	 * @return 平台返回的字符串
	 */
	public String getInterfaceConfig(String interfaceCode){
		String checkKey="MapInterface:"+interfaceCode;
		String interfaceCodeInfo=redisUtil.get(checkKey);
		if ((interfaceCodeInfo=="")||(interfaceCodeInfo==null)){
			//获取不到重新查询上传
			String params="{\"params\":[{\"FindFieldVal\":{}}],\"code\":\"4003\",\"session\":[{}]}";
			ResultSet interfaceResult = WebClient.create()
					.post()
					.uri("http://172.16.2.6:52773/External/hipExternalMutiple")
					.header("Authorization", "Basic cHJoaXA6cHJoaXBAMjAyMA==")
					.body(BodyInserters.fromObject(params))
					.retrieve()
					.bodyToMono(ResultSet.class)
					.block();
			String errorCode=interfaceResult.getErrorCode();
			if ((errorCode!="0")||(errorCode==null)){
				return "";
			}
			List<MapInterface> mapInterfaceArr= (List<MapInterface>) interfaceResult.getData();
			for (MapInterface mapInterface : mapInterfaceArr) {
				String code=mapInterface.getCode();
				String curCheckKye="MapInterface:"+code;
				redisUtil.set(curCheckKye,mapInterface.toString());
			}
			interfaceCodeInfo=redisUtil.get(checkKey);
		}
		return interfaceCodeInfo;
	}

	/**
	 * 解析配置并转发请求
	 * @param postObj
	 * @return
	 */
	@Override
	public Object handerRequest(Object postObj) {
		try {
			//解析Redis

//			Map paramsObj = (Map) postObj;
//			String code= (String) paramsObj.get("code");
//			List<Object> session = (List<Object>) paramsObj.get("session");


			//webClient请求
			Object response=postWebClient(postObj); //webClient 请求方式
			log.info("[线程:{} 操作:{}]",Thread.currentThread().getId(),"客户端模拟发送请求");
			return response;

			//httpRequest请求
//			String postObjStr=JSON.toJSONString(postObj);
//			String responStr=String.valueOf(postHutoolHttps(postObjStr));
//			com.alibaba.fastjson.JSONObject responObj= JSON.parseObject(responStr);  //hutool 请求方式
//			log.info("[线程:{} 操作:{}]",Thread.currentThread().getId(),"客户端模拟发送请求");
//			return responObj;

		} catch (Exception e) {
			e.printStackTrace();
			return null;
		}
	}

	/**
	 * Https转发请求的方法（同步）
	 * @param requestObj
	 * @return
	 */
	public Object postHutoolHttps(String requestObj){
		String url="https://172.16.1.6/bdhealth/";

		Object httpsRes= HttpRequest.post(url)
				.header("Authorization", "Basic cHJoaXA6cHJoaXBAMjAyMA==" )
				.header("content_type", "application/json")
				.body(requestObj)
				.timeout(20000)
				.execute(true).body();
		log.info("[线程:{} 操作:{}]",Thread.currentThread().getId(),"客户端异步收到服务端响应的请求");
		return httpsRes;
	}

	/**
	 * HTTP转发请求POST方法封装(异步)
	 * @return
	 */
	public Mono<Object> postWebClient(Object postObj){
//		String url="https://172.16.1.6/bdhealth/";  //HIS正式库
		String url="http://m00.puruiit.cn:3591/EMRService/sysInternalMutiple";  //EMR测试库
		WebClient webClient = WebClient.builder().baseUrl(url)
				.defaultHeader(HttpHeaders.USER_AGENT, "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36")
//				.defaultHeader(HttpHeaders.AUTHORIZATION, "Basic cHJoaXA6cHJoaXBAMjAyMA==")  //HIS正式库
				.defaultHeader(HttpHeaders.AUTHORIZATION, "Basic cHVydWk6cHIxMjM0")  //EMR测试库
				.defaultHeader(HttpHeaders.CONTENT_TYPE,"application/json")
				.build();
		System.out.println(postObj);
		log.info("[线程:{} 操作:{}]",Thread.currentThread().getId(),"客户端异步收到服务端响应的请求");
		return webClient.post().bodyValue(postObj).retrieve().bodyToMono(Object.class);
	}
}
