package com.purui.ghip.utils;

import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Component;

import java.util.concurrent.TimeUnit;

/**
 * Redis工具类
 *
 * @author charlie
 */
@Component
public class RedisUtil {

	private final StringRedisTemplate redisTemplate;

	public RedisUtil(StringRedisTemplate redisTemplate) {
		this.redisTemplate = redisTemplate;
	}

	/**
	 * 递增键的值，并返回递增后的值
	 * @param key 键
	 * @param delta 递增步长
	 * @return 递增后的值
	 */
	public Long increment(String key, long delta) {
		return redisTemplate.opsForValue().increment(key, delta);
	}

	/**
	 * 设置键的过期时间
	 * @param key 键
	 * @param timeout 过期时间
	 * @param unit 时间单位
	 */
	public void expire(String key, long timeout, TimeUnit unit) {
		redisTemplate.expire(key, timeout, unit);
	}

	/**
	 * 获取键的值
	 * @param key 键
	 * @return 键对应的值
	 */
	public String get(String key) {
		return redisTemplate.opsForValue().get(key);
	}

	/**
	 * 设置键值对
	 * @param key 键
	 * @param value 值
	 */
	public void set(String key, String value) {
		redisTemplate.opsForValue().set(key, value);
	}

	/**
	 * 设置键值对，并设置过期时间
	 * @param key 键
	 * @param value 值
	 * @param timeout 过期时间
	 * @param unit 时间单位
	 */
	public void set(String key, String value, long timeout, TimeUnit unit) {
		redisTemplate.opsForValue().set(key, value, timeout, unit);
	}

	/**
	 * 删除键
	 * @param key 键
	 */
	public void delete(String key) {
		redisTemplate.delete(key);
	}

}